@@ -13,9 +13,12 @@ import tempfile
 import threading
 import time
 
+from hashlib import md5
+from pathlib import Path
 from six.moves import shlex_quote as quote
 from six.moves.urllib.parse import urlparse
 from six.moves.urllib.parse import urlunparse
+from subprocess import check_output
 from unicodedata import normalize
 
 from dallinger import data
@@ -30,6 +33,7 @@ from dallinger.utils import connect_to_redis
 from dallinger.utils import dallinger_package_path
 from dallinger.utils import ensure_directory
 from dallinger.utils import get_base_url
+from dallinger.utils import get_editable_dallinger_path
 from dallinger.utils import open_browser
 from dallinger.utils import GitClient
 from faker import Faker
@@ -76,7 +80,7 @@ class ExperimentFileSource(object):
         """A Set of all files copyable in the source directory, accounting for
         exclusions.
         """
-        return {path for path in self._walk()}
+        return set(self._walk())
 
     @property
     def size(self):
@@ -187,13 +191,20 @@ class ExplicitFileSource(object):
             shutil.copyfile(from_path, to_path)
 
 
-def assemble_experiment_temp_dir(config):
+def assemble_experiment_temp_dir(config, for_remote=False):
     """Create a temp directory from which to run an experiment.
+    If for_remote is set to True the preparation includes bundling
+    the local dallinger version if it was installed in editable mode.
+
     The new directory will include:
     - Copies of custom experiment files which don't match the exclusion policy
     - Templates and static resources from Dallinger
     - An export of the loaded configuration
     - Heroku-specific files (Procile, runtime.txt) from Dallinger
+    - A requirements.txt file with the contents from the constraints.txt file
+      in the experiment (Dallinger should have generated one with pip-compile
+      if needed by the time we reach this code)
+    - A dallinger zip (only if dallinger is installed in editable mode)
 
     Assumes the experiment root directory is the current working directory.
 
@@ -235,9 +246,9 @@ def assemble_experiment_temp_dir(config):
         os.path.join("templates", "error.html"),
         os.path.join("templates", "error-complete.html"),
         os.path.join("templates", "launch.html"),
-        os.path.join("templates", "complete.html"),
         os.path.join("templates", "questionnaire.html"),
-        os.path.join("templates", "thanks.html"),
+        os.path.join("templates", "exit_recruiter.html"),
+        os.path.join("templates", "exit_recruiter_mturk.html"),
         os.path.join("templates", "waiting.html"),
         os.path.join("templates", "login.html"),
         os.path.join("templates", "dashboard_lifecycle.html"),
@@ -278,10 +289,27 @@ def assemble_experiment_temp_dir(config):
 
     ExplicitFileSource(os.getcwd()).selective_copy_to(dst)
 
+    requirements_path = Path(dst) / "requirements.txt"
+    # Overwrite the requirements.txt file with the contents of the constraints.txt file
+    (Path(dst) / "constraints.txt").replace(requirements_path)
+
+    if for_remote:
+        dallinger_path = get_editable_dallinger_path()
+        if dallinger_path:
+            egg_name = build_and_place(dallinger_path, dst)
+            # Replace the line about dallinger in requirements.txt so that
+            # it refers to the just generated package
+            constraints_text = requirements_path.read_text()
+            new_constraints_text = re.sub(
+                "dallinger==.*", f"file:{egg_name}", constraints_text
+            )
+            requirements_path.write_text(new_constraints_text)
     return dst
 
 
-def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
+def setup_experiment(
+    log, debug=True, verbose=False, app=None, exp_config=None, local_checks=True
+):
     """Checks the experiment's python dependencies, then prepares a temp directory
     with files merged from the custom experiment and Dallinger.
 
@@ -290,20 +318,21 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
     """
 
     # Verify that the Postgres server is running.
-    try:
-        db.check_connection()
-    except Exception:
-        log("There was a problem connecting to the Postgres database!")
-        raise
-
-    # Check that the demo-specific requirements are satisfied.
-    try:
-        with open("requirements.txt", "r") as f:
-            dependencies = [r for r in f.readlines() if r[:3] != "-e "]
-    except (OSError, IOError):
-        dependencies = []
-    pkg_resources.require(dependencies)
+    if local_checks:
+        try:
+            db.check_connection()
+        except Exception:
+            log("There was a problem connecting to the Postgres database!")
+            raise
 
+        # Check that the demo-specific requirements are satisfied.
+        try:
+            with open("requirements.txt", "r") as f:
+                dependencies = [r for r in f.readlines() if r[:3] != "-e "]
+        except (OSError, IOError):
+            dependencies = []
+        pkg_resources.require(dependencies)
+    ensure_constraints_file_presence(os.getcwd())
     # Generate a unique id for this experiment.
     from dallinger.experiment import Experiment
 
@@ -334,7 +363,7 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
     if not config.get("dashboard_password", None):
         config.set("dashboard_password", fake.password(length=20, special_chars=False))
 
-    temp_dir = assemble_experiment_temp_dir(config)
+    temp_dir = assemble_experiment_temp_dir(config, for_remote=not local_checks)
     log("Deployment temp directory: {}".format(temp_dir), chevrons=False)
 
     # Zip up the temporary directory and place it in the cwd.
@@ -349,6 +378,70 @@ def setup_experiment(log, debug=True, verbose=False, app=None, exp_config=None):
     return (heroku_app_id, temp_dir)
 
 
+def ensure_constraints_file_presence(directory: str):
+    """Looks into the path represented by the string `directory`.
+    Does nothing if a `constraints.txt` file exists there and is
+    newer than a sibling `requirements.txt` file.
+    If it exists but is not up to date a ValueError exception is raised.
+    Otherwise it creates the constraints.txt file based on the
+    contents of the `requirements.txt` file.
+
+    If the `requirements.txt` does not exist one is created with
+    `dallinger` as its only dependency.
+    """
+    constraints_path = Path(directory) / "constraints.txt"
+    requirements_path = Path(directory) / "requirements.txt"
+    if not requirements_path.exists():
+        requirements_path.write_text("dallinger\n")
+    requirements_path_sha = md5(requirements_path.read_bytes()).hexdigest()
+    if constraints_path.exists():
+        if requirements_path_sha in constraints_path.read_text():
+            return
+        else:
+            raise ValueError(
+                "\nChanges detected to requirements.txt: run the command\n    dallinger generate-constraints\nand retry"
+            )
+
+    prev_cwd = os.getcwd()
+    try:
+        os.chdir(directory)
+        compile_info = f"dallinger generate-constraints\n#\n# Compiled from a requirement.txt file with sha: {requirements_path_sha}"
+        check_output(
+            ["pip-compile", "requirements.txt", "-o", "constraints.txt"],
+            env=dict(
+                os.environ,
+                CUSTOM_COMPILE_COMMAND=compile_info,
+            ),
+        )
+    finally:
+        os.chdir(prev_cwd)
+    # Make the path the experiment requirements.txt file relative
+    constraints_contents = constraints_path.read_text()
+    constraints_contents_amended = re.sub(
+        "via -r .*requirements.txt", "via -r requirements.txt", constraints_contents
+    )
+    constraints_path.write_text(constraints_contents_amended)
+
+
+def build_and_place(source: str, destination: str) -> str:
+    """Builds a python egg with the source found at `source` and places it in
+    `destination`.
+    Only works if dallinger is currently installed in editable mode.
+
+    Returns the full path of the newly created distribution file.
+    """
+    old_dir = os.getcwd()
+    try:
+        os.chdir(source)
+        check_output(["python", "-m", "build"])
+        # The built package is the last addition to the `dist` directory
+        package_path = max(Path(source).glob("dist/*"), key=os.path.getctime)
+        shutil.copy(package_path, destination)
+    finally:
+        os.chdir(old_dir)
+    return package_path.name
+
+
 INITIAL_DELAY = 1
 BACKOFF_FACTOR = 2
 MAX_ATTEMPTS = 6
@@ -559,6 +652,10 @@ class HerokuLocalDeployment(object):
     tmp_dir = None
     dispatch = {}  # Subclass may provide handlers for Heroku process output
     environ = None
+    bot = False
+    DEPLOY_NAME = "Heroku"
+    WRAPPER_CLASS = HerokuLocalWrapper
+    DO_INIT_DB = True
 
     def configure(self):
         self.exp_config.update({"mode": "debug", "loglevel": 0})
@@ -569,6 +666,10 @@ class HerokuLocalDeployment(object):
         )
 
     def update_dir(self):
+        # FIXME: this call is used for implicit communication between classes in this file
+        # and service wrappers (HerokuLocalWrapper, DockerComposeWrapper).
+        # This communication should be made explicit, passing this path around instead of
+        # changing a global state.
         os.chdir(self.tmp_dir)
         # Update the logfile to the new directory
         config = get_config()
@@ -579,21 +680,28 @@ class HerokuLocalDeployment(object):
         config.write()
 
     def run(self):
-        """Set up the environment, get a HerokuLocalWrapper instance, and pass
+        """Set up the environment, get a wrapper instance, and pass
         it to the concrete class's execute() method.
         """
         self.configure()
         self.setup()
         self.update_dir()
-        db.init_db(drop_all=True)
+        if self.DO_INIT_DB:
+            db.init_db(drop_all=True)
         config = get_config()
         environ = None
         if self.environ:
             environ = os.environ.copy()
             environ.update(self.environ)
-        self.out.log("Starting up the Heroku Local server...")
-        with HerokuLocalWrapper(
-            config, self.out, verbose=self.verbose, env=environ
+        self.out.log(f"Starting up the {self.DEPLOY_NAME} Local server...")
+        with self.WRAPPER_CLASS(
+            config,
+            self.out,
+            self.original_dir,
+            self.tmp_dir,
+            verbose=self.verbose,
+            env=environ,
+            needs_chrome=self.bot,
         ) as wrapper:
             try:
                 self.execute(wrapper)
