@@ -161,6 +161,14 @@ class Experiment(object):
         else:
             self.widget = module.ExperimentWidget(self)
 
+    @classmethod
+    def extra_parameters(cls):
+        """Override this classmethod to register new config variables. It is
+        called during config load. See
+        :ref:`Extra Configuration <extra-configuration>` for an example.
+        """
+        pass
+
     def configure(self):
         """Load experiment configuration here"""
         pass
@@ -309,6 +317,25 @@ class Experiment(object):
         """
         network.add_node(node)
 
+    def normalize_entry_information(self, entry_information):
+        """Accepts a dictionary with information about a recruited user. Returns
+        a dictionary containing data the needed to create or load a Dallinger
+        Participant. The returned data should include valid ``assignment_id``,
+        ``worker_id``, and ``hit_id`` values. It may also include an
+        ``entry_information`` key which should contain a transformed
+        ``entry_information`` dict which will be stored for newly created
+        participants.
+
+        By default, the extraction of these values is delegated to the
+        recruiter's `normalize_entry_information` method.
+
+        Returning a dictionary without valid ``hit_id``, ``assignment_id``, or
+        ``worker_id`` will generally result in an exception.
+        """
+        entry_data = self.recruiter.normalize_entry_information(entry_information)
+        # We need an assignment_id in order to create a participant
+        return entry_data
+
     def create_participant(
         self,
         worker_id,
@@ -317,6 +344,7 @@ class Experiment(object):
         mode,
         recruiter_name=None,
         fingerprint_hash=None,
+        entry_information=None,
     ):
         """Creates and returns a new participant object. Uses
         :attr:`~dallinger.experiment.Experiment.participant_constructor` as the
@@ -332,6 +360,10 @@ class Experiment(object):
         :type mode: str
         :param recruiter_name: the recruiter name
         :type recruiter_name: str
+        :param fingerprint_hash: the user's fingerprint
+        :type fingerprint_hash: str
+        :param entry_information: a JSON serializable data structure containing
+                                  additional participant entry information
         :returns: A :attr:`~dallinger.models.Participant` instance
         """
         if not recruiter_name:
@@ -346,6 +378,7 @@ class Experiment(object):
             hit_id=hit_id,
             mode=mode,
             fingerprint_hash=fingerprint_hash,
+            entry_information=entry_information,
         )
         self.session.add(participant)
         return participant
@@ -396,6 +429,28 @@ class Experiment(object):
         """
         return "Thank for participating! Here is your bonus."
 
+    def exit_info_for(self, participant):
+        """An experiment can return a dictionary of infomation that will
+        be shown to the participant at the very last point in their
+        lifecycle, if the HIT is not submitted to an external recruitment
+        service for submission.
+
+        For complete control over the exit page, a customized version of
+        the ``exit_recruiter.html`` template can be included in the experient
+        directory, and this will override the default provided by Dallinger.
+
+        :param participant: the ``Participant`` instance for which to calculate
+            an exit value
+        :returns: ``dict`` which may be rendered to the worker as an HTML table
+            when they submit their assigment.
+        """
+        return {
+            "Assignment ID": participant.assignment_id,
+            "HIT ID": participant.hit_id,
+            "Base Pay": participant.base_pay,
+            "Bonus": participant.bonus,
+        }
+
     def attention_check(self, participant):
         """Check if participant performed adequately.
 
@@ -1008,7 +1063,7 @@ class Experiment(object):
 
             try:
                 extra_parameters()
-                extra_parameters.loaded = True
+                config._module_params_loaded = True
             except KeyError:
                 pass
         except ImportError:
@@ -1074,11 +1129,6 @@ class Experiment(object):
         self.import_session.close()
         session.rollback()
         session.close()
-        # Remove marker preventing experiment config variables being reloaded
-        try:
-            del module.extra_parameters.loaded
-        except AttributeError:
-            pass
         config._reset(register_defaults=True)
         del sys.modules["dallinger_experiment"]
 
@@ -1270,7 +1320,10 @@ def load():
         try:
             from dallinger_experiment import experiment
         except ImportError:
-            from dallinger_experiment import dallinger_experiment as experiment
+            try:
+                from dallinger_experiment import dallinger_experiment as experiment
+            except ImportError:
+                import dallinger_experiment as experiment
 
         classes = inspect.getmembers(experiment, is_experiment_class)
 
