@@ -373,22 +373,6 @@ def launch():
     return success_response(recruitment_msg=message)
 
 
-def should_show_thanks_page_to(participant):
-    """In the context of the /ad route, should the participant be shown
-    the thanks.html page instead of ad.html?
-    """
-    if participant is None:
-        return False
-    status = participant.status
-    marked_done = participant.end_time is not None
-    ready_for_external_submission = (
-        status in ("overrecruited", "working") and marked_done
-    )
-    assignment_complete = status in ("submitted", "approved")
-
-    return assignment_complete or ready_for_external_submission
-
-
 @app.route("/ad", methods=["GET"])
 @nocache
 def advertisement():
@@ -403,8 +387,6 @@ def advertisement():
         These arguments will have appropriate values and we should enter the
         person in the database and provide a link to the experiment popup.
     """
-    if not ("hitId" in request.args and "assignmentId" in request.args):
-        raise ExperimentError("hit_assign_worker_id_not_set_in_mturk")
     config = _config()
 
     # Browser rule validation, if configured:
@@ -412,40 +394,30 @@ def advertisement():
     if not browser.is_supported(request.user_agent.string):
         raise ExperimentError("browser_type_not_allowed")
 
-    hit_id = request.args["hitId"]
-    assignment_id = request.args["assignmentId"]
+    entry_information = request.args.to_dict()
     app_id = config.get("id", "unknown")
-    mode = config.get("mode")
-    debug_mode = mode == "debug"
-    worker_id = request.args.get("workerId")
-    participant = None
+    exp = Experiment(session)
+    entry_data = exp.normalize_entry_information(entry_information)
+
+    hit_id = entry_data.get("hit_id")
+    assignment_id = entry_data.get("assignment_id")
+    worker_id = entry_data.get("worker_id")
+
+    if not (hit_id and assignment_id):
+        raise ExperimentError("hit_assign_worker_id_not_set_by_recruiter")
 
     if worker_id is not None:
-        # First check if this workerId has completed the task before
-        # under a different assignment (v1):
-        already_participated = bool(
+        # Check if this workerId has completed the task before
+        already_participated = (
             models.Participant.query.filter(
-                models.Participant.assignment_id != assignment_id
-            )
-            .filter(models.Participant.worker_id == worker_id)
-            .count()
+                models.Participant.worker_id == worker_id
+            ).first()
+            is not None
         )
 
-        if already_participated and not debug_mode:
+        if already_participated:
             raise ExperimentError("already_did_exp_hit")
 
-        # Next, check for participants already associated with this very
-        # assignment, and retain their status, if found:
-        try:
-            participant = (
-                models.Participant.query.filter(models.Participant.hit_id == hit_id)
-                .filter(models.Participant.assignment_id == assignment_id)
-                .filter(models.Participant.worker_id == worker_id)
-                .one()
-            )
-        except exc.SQLAlchemyError:
-            pass
-
     recruiter_name = request.args.get("recruiter")
     if recruiter_name:
         recruiter = recruiters.by_name(recruiter_name)
@@ -453,23 +425,6 @@ def advertisement():
         recruiter = recruiters.from_config(config)
         recruiter_name = recruiter.nickname
 
-    if should_show_thanks_page_to(participant):
-        # They've either done, or they're from a recruiter that requires
-        # submission of an external form to complete their participation.
-        return render_template(
-            "thanks.html",
-            hitid=hit_id,
-            assignmentid=assignment_id,
-            workerid=worker_id,
-            external_submit_url=recruiter.external_submission_url,
-            mode=config.get("mode"),
-            app_id=app_id,
-        )
-    if participant and participant.status == "working":
-        # Once participants have finished the instructions, we do not allow
-        # them to start the task again.
-        raise ExperimentError("already_started_exp_mturk")
-
     # Participant has not yet agreed to the consent. They might not
     # even have accepted the HIT.
     return render_template(
@@ -480,9 +435,40 @@ def advertisement():
         workerid=worker_id,
         mode=config.get("mode"),
         app_id=app_id,
+        query_string=request.query_string.decode(),
     )
 
 
+@app.route("/recruiter-exit", methods=["GET"])
+@nocache
+def recriter_exit():
+    """Display an exit page defined by the Participant's Recruiter.
+    The Recruiter may in turn delegate to the Experiment for additional
+    values to display.
+    """
+    participant_id = request.args.get("participant_id")
+    if participant_id is None:
+        return error_response(
+            error_type="/recruiter-exit GET: param participant_id is required",
+            status=400,
+        )
+    participant = models.Participant.query.get(participant_id)
+    if participant is None:
+        return error_response(
+            error_type="/recruiter-exit GET: no participant found for ID {}".format(
+                participant_id
+            ),
+            status=404,
+        )
+
+    # Get the recruiter from the participant rather than config, to support
+    # MultiRecruiter experiments
+    recruiter = recruiters.by_name(participant.recruiter_id)
+    exp = Experiment(session)
+
+    return recruiter.exit_response(experiment=exp, participant=participant)
+
+
 @app.route("/summary", methods=["GET"])
 def summary():
     """Summarize the participants' status codes."""
@@ -572,12 +558,21 @@ def get_page_from_directory(directory, page):
 def consent():
     """Return the consent form. Here for backwards-compatibility with 2.x."""
     config = _config()
+
+    entry_information = request.args.to_dict()
+    exp = Experiment(session)
+    entry_data = exp.normalize_entry_information(entry_information)
+
+    hit_id = entry_data.get("hit_id")
+    assignment_id = entry_data.get("assignment_id")
+    worker_id = entry_data.get("worker_id")
     return render_template(
         "consent.html",
-        hit_id=request.args["hit_id"],
-        assignment_id=request.args["assignment_id"],
-        worker_id=request.args["worker_id"],
+        hit_id=hit_id,
+        assignment_id=assignment_id,
+        worker_id=worker_id,
         mode=config.get("mode"),
+        query_string=request.query_string.decode(),
     )
 
 
@@ -674,7 +669,7 @@ def assign_properties(thing):
 
 @app.route("/participant/<worker_id>/<hit_id>/<assignment_id>/<mode>", methods=["POST"])
 @db.serialized
-def create_participant(worker_id, hit_id, assignment_id, mode):
+def create_participant(worker_id, hit_id, assignment_id, mode, entry_information=None):
     """Create a participant.
 
     This route is hit early on. Any nodes the participant creates will be
@@ -747,7 +742,13 @@ def create_participant(worker_id, hit_id, assignment_id, mode):
     # Create the new participant.
     exp = Experiment(session)
     participant = exp.create_participant(
-        worker_id, hit_id, assignment_id, mode, recruiter_name, fingerprint_hash
+        worker_id,
+        hit_id,
+        assignment_id,
+        mode,
+        recruiter_name,
+        fingerprint_hash,
+        entry_information,
     )
 
     session.flush()
@@ -771,6 +772,17 @@ def create_participant(worker_id, hit_id, assignment_id, mode):
     return success_response(**result)
 
 
+@app.route("/participant", methods=["POST"])
+def post_participant():
+    config = _config()
+    entry_information = request.form.to_dict()
+    # Remove the mode from entry_information if provided
+    mode = entry_information.pop("mode", config.get("mode"))
+    exp = Experiment(session)
+    participant_info = exp.normalize_entry_information(entry_information)
+    return create_participant(mode=mode, **participant_info)
+
+
 @app.route("/participant/<participant_id>", methods=["GET"])
 def get_participant(participant_id):
     """Get the participant with the given id."""
@@ -785,21 +797,24 @@ def get_participant(participant_id):
     return success_response(participant=ppt.__json__())
 
 
-@app.route("/participant", methods=["POST"])
+@app.route("/load-participant", methods=["POST"])
 def load_participant():
     """Get the participant with an assignment id provided in the request.
     Delegates to :func:`~dallinger.experiments.Experiment.load_participant`.
     """
-    assignment_id = request_parameter("assignment_id", optional=True)
+    entry_information = request.form.to_dict()
+    exp = Experiment(session)
+    participant_info = exp.normalize_entry_information(entry_information)
+
+    assignment_id = participant_info.get("assignment_id")
     if assignment_id is None:
         return error_response(
-            error_type="/participant POST: no participant found", status=403
+            error_type="/load-participant POST: no participant found", status=403
         )
-    exp = Experiment(session)
     ppt = exp.load_participant(assignment_id)
     if ppt is None:
         return error_response(
-            error_type="/participant POST: no participant found", status=403
+            error_type="/load-participant POST: no participant found", status=403
         )
 
     # return the data
@@ -1520,11 +1535,11 @@ def check_for_duplicate_assignments(participant):
         q.enqueue(worker_function, "AssignmentAbandoned", None, d.id)
 
 
-@app.route("/worker_complete", methods=["GET"])
+@app.route("/worker_complete", methods=["POST"])
 @db.scoped_session_decorator
 def worker_complete():
     """Complete worker."""
-    participant_id = request.args.get("participant_id")
+    participant_id = request.values.get("participant_id")
     if not participant_id:
         return error_response(
             error_type="bad request", error_text="participantId parameter is required"
