@@ -78,6 +78,34 @@ class Recruiter(object):
         """
         raise NotImplementedError
 
+    def normalize_entry_information(self, entry_information):
+        """Accepts data from recruited user and returns data needed to validate,
+        create or load a Dallinger Participant.
+
+        See :func:`~dallinger.experiment.Experiment.create_participant` for
+        details.
+
+        The default implementation extracts ``hit_id``, ``assignment_id``, and
+        ``worker_id`` values directly from the ``entry_information``.
+
+        Returning a dictionary without valid ``hit_id``, ``assignment_id``, or
+        ``worker_id`` will generally result in an exception.
+        """
+        participant_data = {
+            "hit_id": entry_information.pop(
+                "hitId", entry_information.pop("hit_id", None)
+            ),
+            "assignment_id": entry_information.pop(
+                "assignmentId", entry_information.pop("assignment_id", None)
+            ),
+            "worker_id": entry_information.pop(
+                "workerId", entry_information.pop("worker_id", None)
+            ),
+        }
+        if entry_information:
+            participant_data["entry_information"] = entry_information
+        return participant_data
+
     def recruit(self, n=1):
         raise NotImplementedError
 
@@ -89,6 +117,12 @@ class Recruiter(object):
         """A recruiter may provide a means to directly compensate a worker."""
         raise NotImplementedError
 
+    def exit_response(self, experiment, participant):
+        """The recruiter returns an appropriate page on experiment/questionnaire
+        submission.
+        """
+        raise NotImplementedError
+
     def reward_bonus(self, assignment_id, amount, reason):
         """Throw an error."""
         raise NotImplementedError
@@ -137,6 +171,18 @@ class CLIRecruiter(Recruiter):
         super(CLIRecruiter, self).__init__()
         self.config = get_config()
 
+    def exit_response(self, experiment, participant):
+        """Delegate to the experiment for possible values to show to the
+        participant.
+        """
+        exit_info = sorted(experiment.exit_info_for(participant).items())
+
+        return flask.render_template(
+            "exit_recruiter.html",
+            recruiter=self.__class__.__name__,
+            participant_exit_info=exit_info,
+        )
+
     def open_recruitment(self, n=1):
         """Return initial experiment URL list, plus instructions
         for finding subsequent recruitment events in experiemnt logs.
@@ -500,11 +546,20 @@ class MTurkRecruiter(Recruiter):
                 'The value of "mode" must be either "sandbox" or "live"'.format(mode)
             )
 
+    def exit_response(self, experiment, participant):
+        return flask.render_template(
+            "exit_recruiter_mturk.html",
+            hitid=participant.hit_id,
+            assignmentid=participant.assignment_id,
+            workerid=participant.worker_id,
+            external_submit_url=self.external_submission_url,
+        )
+
     @property
     def external_submission_url(self):
         """On experiment completion, participants are returned to
         the Mechanical Turk site to submit their HIT, which in turn triggers
-        notifications to the /notifications route.
+        notifications to the /mturk-sns-listener route.
         """
         if self.is_sandbox:
             return "https://workersandbox.mturk.com/mturk/externalSubmit"
@@ -940,7 +995,7 @@ class BotRecruiter(Recruiter):
             url = "{}/ad?{}".format(base_url, ad_parameters)
             urls.append(url)
             bot = factory(url, assignment_id=assignment, worker_id=worker, hit_id=hit)
-            job = q.enqueue(bot.run_experiment, timeout=self._timeout)
+            job = q.enqueue(bot.run_experiment, job_timeout=self._timeout)
             logger.warning("Created job {} for url {}.".format(job.id, url))
 
         return urls
